@@ -27,10 +27,11 @@
 #include "../../pjsua_app_config.h"
 
 #import "ipjsuaViewController.h"
+#import "Reachability.h"
 
 @implementation ipjsuaAppDelegate
 
-#define THIS_FILE	"ipjsuaAppDelegate.m"
+#define THIS_FILE       "ipjsuaAppDelegate.m"
 
 #define KEEP_ALIVE_INTERVAL 600
 
@@ -39,6 +40,51 @@ static pjsua_app_cfg_t  app_cfg;
 static bool             isShuttingDown;
 static char           **restartArgv;
 static int              restartArgc;
+Reachability            *internetReach;
+
+- (void) updateWithReachability: (Reachability *)curReach
+{
+    NetworkStatus netStatus = [curReach currentReachabilityStatus];
+    BOOL connectionRequired = [curReach connectionRequired];
+    switch (netStatus) {
+        case NotReachable:
+            PJ_LOG(3,("", "Access Not Available.."));
+            connectionRequired= NO;
+            break;
+        case ReachableViaWiFi:
+            PJ_LOG(3,("", "Reachable WiFi.."));
+            break;
+        case ReachableViaWWAN:
+            PJ_LOG(3,("", "Reachable WWAN.."));
+        break;
+    }
+    if (connectionRequired) {
+        PJ_LOG(3,("", "Connection Required"));
+    }
+}
+
+/* Called by Reachability whenever status changes. */
+- (void)reachabilityChanged: (NSNotification *)note
+{
+    Reachability* curReach = [note object];
+    NSParameterAssert([curReach isKindOfClass: [Reachability class]]);
+    PJ_LOG(3,("", "reachability changed.."));
+    [self updateWithReachability: curReach];
+    
+    if ([curReach currentReachabilityStatus] != NotReachable &&
+        ![curReach connectionRequired])
+    {
+        pjsua_ip_change_param param;
+        pjsua_ip_change_param_default(&param);
+        pjsua_handle_ip_change(&param);
+    }
+}
+
+
+void displayLog(const char *msg, int len)
+{
+    NSLog(@"%.*s", len, msg);
+}
 
 static void displayMsg(const char *msg)
 {
@@ -52,11 +98,11 @@ static void pjsuaOnStartedCb(pj_status_t status, const char* msg)
     char errmsg[PJ_ERR_MSG_SIZE];
     
     if (status != PJ_SUCCESS && (!msg || !*msg)) {
-	pj_strerror(status, errmsg, sizeof(errmsg));
-	PJ_LOG(3,(THIS_FILE, "Error: %s", errmsg));
-	msg = errmsg;
+        pj_strerror(status, errmsg, sizeof(errmsg));
+        PJ_LOG(3,(THIS_FILE, "Error: %s", errmsg));
+        msg = errmsg;
     } else {
-	PJ_LOG(3,(THIS_FILE, "Started: %s", msg));
+        PJ_LOG(3,(THIS_FILE, "Started: %s", msg));
     }
 
     displayMsg(msg);
@@ -68,12 +114,12 @@ static void pjsuaOnStoppedCb(pj_bool_t restart,
     PJ_LOG(3,("ipjsua", "CLI %s request", (restart? "restart" : "shutdown")));
     if (restart) {
         displayMsg("Restarting..");
-	pj_thread_sleep(100);
+        pj_thread_sleep(100);
         app_cfg.argc = argc;
         app_cfg.argv = argv;
     } else {
         displayMsg("Shutting down..");
-	pj_thread_sleep(100);
+        pj_thread_sleep(100);
         isShuttingDown = true;
     }
 }
@@ -95,11 +141,11 @@ static void pjsuaOnAppConfigCb(pjsua_app_config *cfg)
     
     pj_bzero(&app_cfg, sizeof(app_cfg));
     if (restartArgc) {
-	app_cfg.argc = restartArgc;
-	app_cfg.argv = restartArgv;
+        app_cfg.argc = restartArgc;
+        app_cfg.argv = restartArgv;
     } else {
-	app_cfg.argc = argc;
-	app_cfg.argv = (char**)argv;
+        app_cfg.argc = argc;
+        app_cfg.argv = (char**)argv;
     }
     app_cfg.on_started = &pjsuaOnStartedCb;
     app_cfg.on_stopped = &pjsuaOnStoppedCb;
@@ -116,11 +162,14 @@ static void pjsuaOnAppConfigCb(pjsua_app_config *cfg)
         }
     
         /* Setup device orientation change notification */
-        [[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
-        [[NSNotificationCenter defaultCenter] addObserver:app
-            selector:@selector(orientationChanged:)
-            name:UIDeviceOrientationDidChangeNotification
-            object:[UIDevice currentDevice]];
+        dispatch_async(dispatch_get_main_queue(),
+        ^{
+            [[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
+            [[NSNotificationCenter defaultCenter] addObserver:app
+                selector:@selector(orientationChanged:)
+                name:UIDeviceOrientationDidChangeNotification
+                object:[UIDevice currentDevice]];
+        });
         
         status = pjsua_app_run(PJ_TRUE);
         if (status != PJ_SUCCESS) {
@@ -150,6 +199,17 @@ static void pjsuaOnAppConfigCb(pjsua_app_config *cfg)
     self.window.rootViewController = self.viewController;
     [self.window makeKeyAndVisible];
     
+    /* Observe the kNetworkReachabilityChangedNotification. When that
+     * notification is posted, the method "reachabilityChanged" will be called.
+     */
+    [[NSNotificationCenter defaultCenter] addObserver: self
+          selector: @selector(reachabilityChanged:)
+          name: kReachabilityChangedNotification object: nil];
+    
+    internetReach = [Reachability reachabilityForInternetConnection];
+    [internetReach startNotifier];
+    [self updateWithReachability: internetReach];
+    
     app = self;
     
     /* Start pjsua app thread */
@@ -180,10 +240,11 @@ static void pjsuaOnAppConfigCb(pjsua_app_config *cfg)
     static pj_thread_t *a_thread;
     static UIDeviceOrientation prev_ori = 0;
     UIDeviceOrientation dev_ori = [[UIDevice currentDevice] orientation];
+    int i;
     
     if (dev_ori == prev_ori) return;
     
-    NSLog(@"Device orientation changed: %d", (prev_ori = dev_ori));
+    NSLog(@"Device orientation changed: %d", (int)(prev_ori = dev_ori));
     
     if (dev_ori >= UIDeviceOrientationPortrait &&
         dev_ori <= UIDeviceOrientationLandscapeRight)
@@ -192,9 +253,15 @@ static void pjsuaOnAppConfigCb(pjsua_app_config *cfg)
             pj_thread_register("ipjsua", a_thread_desc, &a_thread);
         }
         
-        pjsua_vid_dev_set_setting(PJMEDIA_VID_DEFAULT_CAPTURE_DEV,
-                                  PJMEDIA_VID_DEV_CAP_ORIENTATION,
-                                  &pj_ori[dev_ori-1], PJ_TRUE);
+        /* Here we set the orientation for all video devices.
+         * This may return failure for renderer devices or for
+         * capture devices which do not support orientation setting,
+         * we can simply ignore them.
+         */
+        for (i = pjsua_vid_dev_count()-1; i >= 0; i--) {
+            pjsua_vid_dev_set_setting(i, PJMEDIA_VID_DEV_CAP_ORIENTATION,
+                                      &pj_ori[dev_ori-1], PJ_TRUE);
+        }
     }
 #endif
 }
@@ -205,7 +272,7 @@ static void pjsuaOnAppConfigCb(pjsua_app_config *cfg)
     int i;
     
     if (!pj_thread_is_registered()) {
-	pj_thread_register("ipjsua", a_thread_desc, &a_thread);
+        pj_thread_register("ipjsua", a_thread_desc, &a_thread);
     }
     
     /* Since iOS requires that the minimum keep alive interval is 600s,
@@ -224,9 +291,13 @@ static void pjsuaOnAppConfigCb(pjsua_app_config *cfg)
     // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
     // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
     [self performSelectorOnMainThread:@selector(keepAlive) withObject:nil waitUntilDone:YES];
+
+#if 0
+    /* setKeepAliveTimeout is deprecated. Use PushKit instead. */
     [application setKeepAliveTimeout:KEEP_ALIVE_INTERVAL handler: ^{
-	[self performSelectorOnMainThread:@selector(keepAlive) withObject:nil waitUntilDone:YES];
+        [self performSelectorOnMainThread:@selector(keepAlive) withObject:nil waitUntilDone:YES];
     }];
+#endif
 }
 
 - (void)applicationWillEnterForeground:(UIApplication *)application
@@ -247,21 +318,28 @@ static void pjsuaOnAppConfigCb(pjsua_app_config *cfg)
 
 pj_bool_t showNotification(pjsua_call_id call_id)
 {
+    /* This is deprecated. Use VoIP Push Notifications with PushKit
+     * framework instead.
+     */
+#if 0
     // Create a new notification
     UILocalNotification* alert = [[UILocalNotification alloc] init];
     if (alert)
     {
-	alert.repeatInterval = 0;
-	alert.alertBody = @"Incoming call received...";
+        alert.repeatInterval = 0;
+        alert.alertBody = @"Incoming call received...";
         /* This action just brings the app to the FG, it doesn't
          * automatically answer the call (unless you specify the
          * --auto-answer option).
          */
-	alert.alertAction = @"Activate app";
-	
-	[[UIApplication sharedApplication] presentLocalNotificationNow:alert];
+        alert.alertAction = @"Activate app";
+        
+        dispatch_async(dispatch_get_main_queue(),
+                       ^{[[UIApplication sharedApplication]
+                          presentLocalNotificationNow:alert];});
     }
-    
+#endif
+
     return PJ_FALSE;
 }
 
@@ -274,14 +352,14 @@ void displayWindow(pjsua_vid_win_id wid)
     last = (wid == PJSUA_INVALID_ID) ? PJSUA_MAX_VID_WINS : wid+1;
 
     for (;i < last; ++i) {
-	pjsua_vid_win_info wi;
+        pjsua_vid_win_info wi;
         
         if (pjsua_vid_win_get_info(i, &wi) == PJ_SUCCESS) {
-            UIView *parent = app.viewController.view;
-            UIView *view = (__bridge UIView *)wi.hwnd.info.ios.window;
+            dispatch_async(dispatch_get_main_queue(), ^{
+                UIView *parent = app.viewController.view;
+                UIView *view = (__bridge UIView *)wi.hwnd.info.ios.window;
             
-            if (view) {
-                dispatch_async(dispatch_get_main_queue(), ^{
+                if (view) {
                     /* Add the video window as subview */
                     if (![view isDescendantOfView:parent])
                         [parent addSubview:view];
@@ -301,8 +379,8 @@ void displayWindow(pjsua_vid_win_id wid)
                                                   parent.bounds.size.height-
                                                   view.bounds.size.height/2.0);
                     }
-                });
-            }
+                }
+            });
         }
     }
 
